/* ********** Please don't edit this file! */ 
/* ********** It has been generated automatically by dae_devops version 0.5.1. */ 
/* ********** For repository_name soakdb3 */ 

/* The theme normally has this, but I think it's ok to use the full width of the window in all @media sizes.

@media (min-width: 960px)
.bd-page-width {
    max-width: 88rem;
}

*/

.bd-page-width {
    max-width: 100%;
}

/* dae_devops_fingerprint 46fbed06ac7c08af19e0cea761486558 */ 
